####################
# Copyright (c) 2009, Perceptive Automation, LLC. All rights reserved.
# http://www.perceptiveautomation.com
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#
# IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
####################

####################
## IMPORTS
import cherrypy
from cherrypy import _cperror

from indigopy import indigoconn as ic
from indigopy import indigodb as idb
from indigopy.basereqhandler import BaseRequestHandler, kTrueStr, kFalseStr, kEmptyStr, kTextPageStr, kHtmlPageStr, kXmlPageStr
#import cgitb; cgitb.enable()
#from custom_colors import *
import socket
import os
import os.path
import sys
import csv
import glob
import cgi

####################
## CONSTANTS

####################
# Optional hooks to provide a plugin name and description (shown in Event Log and plugin index)
def PluginName():
	return u"Event Log Plugin"

def PluginDescription():
	return u"Displays recent Event Log files."

def ShowOnControlPageList():
	return True		# if True, then above name/description is shown on the Control Page list index

# Optional hook called when the IndigoWebServer first connects to IndigoServer
def IndigoConnected():
	pass

# Optional hook called when the IndigoWebServer disconnect from IndigoServer
def IndigoDisconnected():
	pass

####################
def _read_indigo_log_file(filename, csv_data=[], tag_data=[], dates=[]):
	#
	#		Read in the Indigo Log file (FQPN in filename, and append the CSV, Tag, and dates to the Lists passed into the function
	#
	filename = filename.strip()
	csv_file = csv.reader(open(filename, "rU"), delimiter = '\t')
	for log_line in csv_file:
		if len(log_line) == 3:
			timestamp = log_line[0];
			logtype = log_line[1];
			logdesc = log_line[2];
		elif len(log_line) == 1:
			timestamp = ""
			logtype = ""
			logdesc = log_line[0];
		logdesc = cgi.escape ( logdesc )

		if timestamp <> "" and logtype<> "":
			csv_data.append ( (timestamp.strip(), logtype.strip(), logdesc.strip()) )
		if logtype != "" and tag_data.count (logtype.strip() ) == 0:
			tag_data.append ( logtype.strip() )
		if timestamp != "" and dates.count ( timestamp.split(" ")[0]) == 0:
			dates.append ( timestamp.split(" ")[0] )

		#
		#		Optimize, & move for later versions...
		#		Issue can arise where we are loading multiple files, and thus re-sorting the data each time a new file is loaded
		#		Move to after the file load loop, and validate.
		#
	if sys.version_info[:3] >= (2,4,0):
		tag_data.sort( reverse = True)
		csv_data.sort ( reverse = True)
	else:
		tag_data.reverse ()
		csv_data.reverse ()
		
	return (csv_data, tag_data, dates)

####################
# The functions in this class are automatically called based on the URL
# requested. This provides a mechanism for easily serving dynamic content.
#
# The URL path is based on the folder path of the plugin. For example,
# this plugin folder name is "sample" so any URL path containing "sample"
# as the first path identifier will call into these functions.
#
# The class name can be called whatever you want, but it must subclass
# from BaseRequestHandler to be loaded by the webserver plugin manager.
class HaloHomeRequestHandler(BaseRequestHandler):
	""" Handles HTTP page requests. """

	####################
	# The plugin page request handler must subclass from BaseRequestHandler
	# and must call the parent class __init__ method.
	def __init__(self, logFunc, debugLogFunc):
		BaseRequestHandler.__init__(self, logFunc, debugLogFunc)


	def form_top ( self, html_elems, numofdays, dates, tags, selected_tag="All" ):
		#
		#		Contains the top of the report
		#		Split from the index function to clarify / simplify the visual clutter
		#
		if selected_tag =="" or selected_tag == None:
			selected_tag = "All"
		html_elems.append ("<head>\n")
		html_elems.append("<title>Indigo log viewer</title></Head>\n")
		html_elems.append('<link rel="stylesheet" type="text/css" href="css/EventLog.css">')
		html_elems.append("<body>\n")
		
		html_elems.append ('<table border=0 width="95%"> <TR>')
		html_elems.append('<a name="top"></a>\n')
		html_elems.append ('<form method="post" action="">')
		html_elems.append ("<td>")
		html_elems.append ('Number of Days to View - <select name="numofdays" size="1">')
		for x in range(1, 10):
			if numofdays==str(x):
				html_elems.append ('<option value="%s" SELECTED>%s' % (str(x), str(x) ) )
			else:
				html_elems.append ('<option value="%s">%s' % (str(x), str(x) ) )
		html_elems.append ('</select>')
		html_elems.append ('<input type="submit">')
		html_elems.append ("</td>")
		html_elems.append ("<td>")
		html_elems.append ('View a Particular Day - <select name="date_selection" size="1">')
		html_elems.append ('<option>All')
		for day in dates:
			html_elems.append ('<option>%s'% day)
		html_elems.append ('</select>')
		html_elems.append ('<input type="submit">')
		html_elems.append ("</td>")
		html_elems.append ("<td>")
		html_elems.append ('Filter by Entry Type - <select name="tag_selection" size="1">')
		html_elems.append ('<option SELECTED>%s' % selected_tag)
		html_elems.append ('<option>All')
		for tag in tags:
			html_elems.append ('<option>%s'% tag)
		html_elems.append ('</select>')
		html_elems.append ('<input type="submit">')
		html_elems.append ("</td></tr> </table>\n")

		return html_elems

	####################
	def index(self, numofdays=2, date_selection="All", tag_selection=""):
				#
				#
		if os.path.exists (r'/Library/Application Support/Perceptive Automation/Indigo 4/Logs'):
			indigo_path = r'/Library/Application Support/Perceptive Automation/Indigo 4/Logs'
		else:
			indigo_path = r'/Library/Application Support/Perceptive Automation/Indigo 2/Logs'
				
		cherrypy.response.headers['Content-Type'] = 'text/html'
		if date_selection == "All":
			#
			#		List all files in the Indigo Log File Directory
			#
			log_files = glob.glob (indigo_path+'/*')
			log_files = log_files[0-int(numofdays):len(log_files)] # Only load the last X log files (x = numofdays)
			#
			#		Only load numdays worth of files, starting at the end of the list
			#
			if sys.version_info[:3] >= (2,4,0):
				log_files.sort( reverse=True)
			else:
				log_files.reverse ()
				
		else:
			#
			#		Load only the log file that was requested.
			#
			log_files = glob.glob (indigo_path+'/%s*' % date_selection)
		
		csv_data = []
		tag_data = []
		dates = []
		for filename in log_files:
			#
			#		Load each log file into the buffer
			#
			csv_data, tag_data, dates = _read_indigo_log_file(r'%s' % filename, csv_data, tag_data, dates)
		
		html_elems = []
		html_elems = self.form_top ( html_elems, numofdays, dates, tag_data, tag_selection)
		
		html_elems.append("<html>\n")
		current_day = ""
		last_timestamp = ""
		for (timestamp, tag, text) in csv_data:
	
			if text.count ('"') >= 2:
					start_text = text.find ('"')
					end_text = text.find ('"', start_text+1)
					text_length = len(text)
					text = text[0:start_text] + "<B>" + text[start_text:end_text] + "</B>" + text[end_text:text_length]
	
 			clean_text = text.strip()
 			clean_tag = tag.strip()
			clean_tag_css = tag.strip().replace(" ","").replace("/", "").replace("(","").replace(")","")
							
			if timestamp != "" and current_day <> timestamp.strip().split (" ")[0]:
				#
				#		The end of a day has been detected, close table, and open a new HTML table
				#
				if current_day <> "":
					html_elems.append("		</table><p><p>\n")

				current_day = timestamp.strip().split (" ")[0]
				html_elems.append("		 <center><hr> <h3> Log Entries for %s </h3><hr></center>\n" % current_day)
				html_elems.append("		 <a href='#top'>Jump to Top</a><br>\n")
			
				html_elems.append ("	<table border=1 cellpadding=1 cellspacing=0>")

				if timestamp == last_timestamp:
					timestamp = ""
				else:
					timestamp = '<a name="%s">%s</A>' % (current_day, timestamp)

			if timestamp == last_timestamp:
				timestamp = ""

			if tag_selection == "All" or tag_selection=="":
				text_entry = '		 <TR><TD width="17%%" class="DateTimeStamp">%s</TD><TD width="15%%" align=center class="%s">%s</TD><TD class="%s">%s</TD></TR>\n' % (timestamp, clean_tag_css, tag, clean_tag_css, text)
				html_elems.append( text_entry.decode ('utf-8'))
			elif tag_selection == clean_tag:
				text_entry = '		 <TR><TD width="17%%" class="DateTimeStamp">%s</TD><TD width="15%%" align=center class="%s">%s</TD><TD class="%s">%s</TD></TR>\n' % (timestamp, clean_tag_css, tag, clean_tag_css, text)
#				text_entry = '		 <TR><TD width="17%%" class="DateTimeStamp">%s</TD><TD width="15%%" align=center class="%s">%s</TD><TD class="%s">%s</TD></TR>\n' % (timestamp, clean_tag, tag, clean_tag, text)
				html_elems.append( text_entry.decode ('utf-8'))


#				text_entry = '		 <TR><TD width="17%%" class="DateTimeStamp">%s</TD><TD width="15%%" align=center class="%s">%s</TD><TD class="%s">%s</TD></TR>\n' % (timestamp, clean_tag, tag, clean_tag, text)

			#
			#	Add to web page
			#	

			if timestamp <> "": last_timestamp = timestamp
		html_elems.append("	   </table>\n")
		html_elems.append ("</form>")
		html_elems.append("</body>\n")
		
		return ''.join(html_elems)
	index.exposed = True

#
#		Version History - 
#		1.50	- Added External CSS Support
#		1.35	- Table Header revision
#				- Fixed bug with loading a specific day
#				- Added Ability to restrict to particular event tags (eg. iTunes, Application, Web Server)
#
#		1.30	- Table Header revision
#				- Added Custom color support
#				
#		1.21	- Fixed issues with Sort / Reverse with Python v2.4x
#		1.20	- Added Select Form features
#				- Added Color coding for Error messages
#				- Now displayed in newest to oldest order

#		1.00	- Public release as Indigo Plugin
#
